//--------------------------------------------------------------------------------------
// File: BasicHLSL.fx
//
// The effect file for the BasicHLSL sample.  
// 
// Copyright (c) LAB. All rights reserved.
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
texture g_MeshTexture;              // Color texture for mesh

float4 g_diffuse;
float3 g_ambient;
float3 g_specular;
float g_power;
float3 g_emissive;

int g_nNumLights;
float3 g_LightDir[3];               // Light's direction in world space
float4 g_LightDiffuse[3];           // Light's diffuse color
float3 g_LightAmbient;              // Light's ambient color

float    g_fTime;                   // App's time in seconds
float4x4 g_mWorld;                  // World matrix for object
float4x4 g_mWVP;    // World * View * Projection matrix
float3	 g_EyePos;

//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <g_MeshTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 
    float4 Diffuse    : COLOR0;     // vertex diffuse color (note that COLOR0 is clamped from 0..1)
	float3 Specular	  : TEXCOORD0;
    float2 TextureUV  : TEXCOORD1;  // vertex texture coords 
};


//--------------------------------------------------------------------------------------
// This shader computes standard transform and lighting
//--------------------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, 
                         float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0,
                         uniform int nNumLights )
{
    VS_OUTPUT Output;
    float3 wPos;
	wPos = mul( vPos, (float3x3)g_mWorld );

    Output.Position = mul(vPos, g_mWVP);
    
    float3 wNormal;
    wNormal = normalize(mul(vNormal, (float3x3)g_mWorld)); // normal (world space)
    
    float3 totaldiffuse = float3(0,0,0);
    float3 totalspecular = float3(0,0,0);
	float calcpower = g_power * 0.1f;

    for(int i=0; i<nNumLights; i++ ){
		float nl;
		float3 h;
		float nh;	
		float4 tmplight;
		
		nl = dot( wNormal, g_LightDir[i] );
		h = normalize( ( g_LightDir[i] + g_EyePos - wPos ) * 0.5f );
		nh = dot( wNormal, h );

        totaldiffuse += g_LightDiffuse[i] * max(0,dot(wNormal, g_LightDir[i]));
		totalspecular +=  ((nl) < 0) || ((nh) < 0) ? 0 : ((nh) * calcpower);
	}

    Output.Diffuse.rgb = g_diffuse.rgb * totaldiffuse.rgb 
		+ g_ambient + g_emissive;   
    Output.Diffuse.a = g_diffuse.a; 
    
	Output.Specular = g_specular * totalspecular;

    Output.TextureUV = vTexCoord0; 
    
    return Output;    
}


//--------------------------------------------------------------------------------------
// Pixel shader output structure
//--------------------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};


//--------------------------------------------------------------------------------------
// This shader outputs the pixel's color by modulating the texture's
//       color with diffuse material color
//--------------------------------------------------------------------------------------
PS_OUTPUT RenderScenePSTex( VS_OUTPUT In ) 
{ 
    PS_OUTPUT Output;
    Output.RGBColor = tex2D(MeshTextureSampler, In.TextureUV) * In.Diffuse + float4( In.Specular, 0 );
    return Output;
}

PS_OUTPUT RenderScenePSNotex( VS_OUTPUT In ) 
{ 
    PS_OUTPUT Output;
    Output.RGBColor = In.Diffuse + float4( In.Specular, 0 );
    return Output;
}

//--------------------------------------------------------------------------------------
// Renders scene to render target
//--------------------------------------------------------------------------------------
technique RenderL1
{
    pass P0
    {          
        VertexShader = compile vs_3_0 RenderSceneVS( 1 );
        PixelShader  = compile ps_3_0 RenderScenePSTex();
    }
    pass P1
    {          
        VertexShader = compile vs_3_0 RenderSceneVS( 1 );
        PixelShader  = compile ps_3_0 RenderScenePSNotex();
    }

}

technique RenderL2
{
    pass P0
    {          
        VertexShader = compile vs_3_0 RenderSceneVS( 2 );
        PixelShader  = compile ps_3_0 RenderScenePSTex();
    }
    pass P1
    {          
        VertexShader = compile vs_3_0 RenderSceneVS( 2 );
        PixelShader  = compile ps_3_0 RenderScenePSNotex();
    }

}

technique RenderL3
{
    pass P0
    {          
        VertexShader = compile vs_3_0 RenderSceneVS( 3 );
        PixelShader  = compile ps_3_0 RenderScenePSTex();
    }
    pass P1
    {          
        VertexShader = compile vs_3_0 RenderSceneVS( 3 );
        PixelShader  = compile ps_3_0 RenderScenePSNotex();
    }

}
